from urllib.parse import urlencode

from django.shortcuts import get_object_or_404
from django.urls import reverse, reverse_lazy
from django.db.models import Q
from django.views.generic import (
    RedirectView,
    ListView,
    DetailView,
    CreateView,
    UpdateView,
    DeleteView
)
from web.forms import ArticleModelForm, SearchForm, ArticleCommentModelForm
from web.models import Article, Comment


class ArticleIndexView(ListView):
    template_name = 'index.html'
    context_object_name = 'articles'
    model = Article
    ordering = ['-created_at']
    paginate_by = 3

    def dispatch(self, request, *args, **kwargs):
        self.form = self.get_search_form()
        self.search_value = self.get_search_value()
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, *, object_list=None, **kwargs):
        context = super().get_context_data(**kwargs)
        context['form'] = self.form

        if self.search_value:
            context['query'] = urlencode({'search': self.search_value})

        return context

    def get_queryset(self):
        qs = super().get_queryset()
        if self.search_value:
            query = Q(title__icontains=self.search_value) | Q(author__icontains=self.search_value)
            qs = qs.filter(query)

        return qs

    def get_search_form(self):
        return SearchForm(self.request.GET)

    def get_search_value(self):
        if self.form.is_valid():
            return self.form.cleaned_data.get('search')


class MainPageRedirectView(RedirectView):
    pattern_name = 'main_page'


class ArticleCreateView(CreateView):
    template_name = 'article_create.html'
    model = Article
    form_class = ArticleModelForm

    def get_success_url(self):
        return reverse('articles-detail', kwargs={'id': self.object.id})


class ArticleCommentCreateView(CreateView):
    model = Comment
    form_class = ArticleCommentModelForm

    def dispatch(self, request, *args, **kwargs):
        self.article = get_object_or_404(Article, id=self.kwargs.get('article_id'))
        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        form.instance.article = self.article
        return super().form_valid(form)

    def get_success_url(self):
        return reverse('articles-detail', kwargs={'id': self.article.id})


class ArticleDetailView(DetailView):
    template_name = 'article_detail.html'
    model = Article
    context_object_name = 'article'
    pk_url_kwarg = 'id'
    extra_context = {'form': ArticleCommentModelForm}

    def get_context_data(self, **kwargs):
        return super().get_context_data(
            comments=self.object.comments.order_by('-created_at'),
            **kwargs
        )


class ArticleUpdateView(UpdateView):
    model = Article
    template_name = 'update.html'
    form_class = ArticleModelForm
    context_object_name = 'article'
    pk_url_kwarg = 'id'

    def get_success_url(self):
        return reverse('articles-detail', kwargs={'id': self.object.id})


class ArticleDeleteView(DeleteView):
    model = Article
    template_name = 'delete.html'
    context_object_name = 'article'
    success_url = reverse_lazy('main_page')
    pk_url_kwarg = 'id'
